#####################################################################
# Contains storage space for global variables
#####################################################################

from bpy import *
from .settings import *
from . import store
import subprocess, time, os

# initialization
def init():
	context.user_preferences.themes[0].view_3d.back = [0.7,0.7,0.7]				# better visibility
	context.user_preferences.edit.keyframe_new_interpolation_type = 'LINEAR'	# better motion
	context.user_preferences.view.show_view_name = False						# cleaner interface
	context.user_preferences.view.show_mini_axis = False						# cleaner interface
	context.user_preferences.view.use_rotate_around_active = True				# easier to navigate
	#context.user_preferences.system.use_vertex_buffer_objects = True			# VBO gives much better viewport performance
	


# select obj and make it active, deselect everything else
def select(obj):
	try:
		ob=data.objects[obj]
		ops.object.select_all(action="DESELECT")
		ob.select = True
		context.scene.objects.active=ob
	except:
		return None
	else:
		return ob


# validate and get the number of models in the BBImportOrder string
def getNumModel():
	scene = context.scene
	try:
		list = [int(model) for model in scene.BBImportOrder.split(',')]
		return len(list)
	except:
		return -1


# Set the draw mode of the viewport
def setViewShading(drawtype):
	for area in data.window_managers['WinMan'].windows[0].screen.areas:
		if area.type == "VIEW_3D":
			area.spaces[0].viewport_shade = drawtype



# delete EP related objects
def cleanEPObjs(deletionList = None):
	ops.object.select_all(action="DESELECT")
	# use deletionList if supplied
	if deletionList:
		for obj in deletionList:
			obj.select = True
	# otherwise delete everything in EPOBJ list
	else:
		for list in store.epOBJ:
			for obj in list:
				obj.select = True
		store.epOBJ = []
	# call delete operator
	ops.object.delete()


# render a single frame and save
def renderSingle(i):
	context.scene.frame_end = i
	context.scene.frame_start = i
	ops.render.render(animation = True)



# return center of atom cluster
def atomCentre():
	sum = [0,0,0]
	n = 0
	for ob in data.objects:
		if ob.BBInfo != "":
			sum[0] += ob.location[0]
			sum[1] += ob.location[1]
			sum[2] += ob.location[2]
			n += 1
	return [i/n for i in sum]


# launch app in separate process, for better performance on multithreaded computers
def launch(exeName, async = False):
	# try to hide window (does not work recursively)
# 	startupinfo = subprocess.STARTUPINFO()
# 	print("is this startup info the problem?")
# 	startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
# 	startupinfo.wShowWindow = subprocess.SW_HIDE
	#print("just entered launch")
	args=exeName.split(" ")
	print(args)
	if async:
# 		# if running in async mode, return (the process object) immediately
		return subprocess.Popen(args, bufsize=8192, startupinfo=None)
	else:
# 		# otherwise wait until process is finished (and return None)
		subprocess.call(args, bufsize=8192, startupinfo=None)
		return None
	print("just exited launch")
# Wait until process finishes
def wait(process):
	while process.poll() == None:
		time.sleep(0.1)
		# needed if io mode is set to subprocess.PIPE to avoid deadlock
		# process.communicate()



# returns absolute path from relative path
absPath = path.abspath
